# key chars

COMMENT_MARK = "#"
SECTION_IN = "["
SECTION_OUT = "]"
PARSER_SECTION = "@"
BLANK = " "
EQ = "="
ARROW = ">"

DICT_ENTRY = "@dict"
LIST_ENTRY = "@list"
JSON_ENTRY = "@json"

# section types codes
SECTION_TYPES = dict(
    STANDARD=0,
    DIRTREE=1,
    SYMLINKS=2,
    JSON=3
)
