from setuptools import setup

setup(name='big_config',
      version='0.3.1',
      description='Handles a more complex configuration file structure',
      url='http://github.com/vladblindu/',
      author='Vlad Blindu',
      author_email='vlad.blindu@bbsmedia.ro',
      license='BSD',
      packages=['big_config'],
      zip_safe=False)
